function run_task(task_name::String, task_info::Dict,  dependencies_info::Dict, results_connection::MySQL.Connection, completion_status::Dict{String, Bool}, semaphore::Channel{Bool})
    # in case of need process parameters dictionary to get results from dependencies in database
    parameters_dictionary = process_task_parameters(task_info["parameters_dict"], dependencies_info, results_connection)

    try
        task_work(task_info["function"], task_info["arguments"], parameters_dictionary, results_connection)
        println("Completed task $task_name")
        # Update completion status
        completion_status[task_name] = true
    finally
        # Release the token back to the semaphore, regardless of success or failure
        put!(semaphore, true)
    end
end

function task_work(functionName::String, arguments::String, parametersDict::Dict, results_connection::MySQL.Connection)
    # check if the table already exists in database
    df = get_table(results_connection, functionName, arguments, parametersDict)

    if isempty(df)
        # get the dataframe with results
        df = eval(Meta.parse("$functionName($arguments, $parametersDict)"))
        # store the dataframe in the results container
        write_table(results_connection, functionName, arguments, parametersDict, df)
    end

end

function process_task_parameters(original_dict::Dict, dependencies_info::Dict, results_connection::MySQL.Connection)
    # Define the regular expression pattern to match the reference format
    ref_pattern = r"__REF__(\w+)\[!, :(\w+)\]"

    # Initialize a new dictionary to hold the processed parameters
    new_dict = Dict()

    for (key, value) in original_dict
        if occursin("__REF__", value)
            # Extract task name and column name from the reference
            match_result = match(ref_pattern, value)
            if match_result !== nothing
                task_name = match_result.captures[1]
                column_name = Symbol(match_result.captures[2])

                # Retrieve the DataFrame using the task name
                df = get_task_df(task_name, dependencies_info, results_connection)

                # Check if the DataFrame and the column exist
                if df !== nothing && column_name in names(df)
                    # Update the new dictionary with the column value
                    new_dict[key] = df[!, column_name]
                else
                    error("DataFrame or column not found for reference: $value")
                end
            else
                error("Invalid reference format in value: $value")
            end
        else
            # For non-reference values, just copy them to the new dictionary
            new_dict[key] = value
        end
    end

    return new_dict
end

function get_task_df(task_name::String, dependencies_info::Dict, results_connection::MySQL.Connection)
    df = DataFrame()

    if haskey(dependencies_info, task_name)
        task_definition = dependencies_info[task_name]
    
        # Extract the required information
        functionName = get(task_definition, "function", "")  # Default to empty string if not found
        arguments = get(task_definition, "arguments", "")  # Adjust based on actual key if different
        parametersDict = get(task_definition, "parameters_dict", Dict())

        df = get_table(results_connection, functionName, arguments, parametersDict)
    end

    return df
end